#!/usr/bin/env python

import sys
import os
import json
import argparse
import shutil
import glob
import re

from ansurr import rci_nef
from ansurr import rigidipy
from ansurr import compare

from ansurr.functions import check_quiet_print
from datetime import date

sys.tracebacklimit = 0

natsort = lambda s: [int(t) if t.isdigit() else t.lower() for t in re.split('(\d+)', s)]

def main():

    parser = argparse.ArgumentParser()
    parser.add_argument("-p", "--pdb", type=str, help="input PDB file",required=True)
    parser.add_argument("-s", "--shifts", type=str, help="input shifts file in NEF format",required=True)
    parser.add_argument("-l", "--lig", help=" include free ligands when computing flexibility", action="store_true")
    parser.add_argument("-n", "--nonstd", help=" include non-standard residues when computing flexibility", action="store_true")
    parser.add_argument("-o", "--olig", help="combine chains into a single structure when calculating flexibility", action="store_true")
    parser.add_argument("-q", "--quiet", help="supress output to the terminal", action="store_true")
    parser.add_argument("-r", "--reref", help="re-reference shifts using PANAV before calculating RCI (requires Java)",action="store_true")
    parser.add_argument("-v", "--version", action='version', version='\nANSURR | Accuracy of NMR Structures Using RCI and Rigidity v0.0.16 | citation: https://doi.org/10.1038/s41467-020-20177-1')
    args = parser.parse_args()

    path_to_shifts = args.shifts
    path_to_pdb = args.pdb # need to check that file looks like a PDB file
    
    shift_file = os.path.basename(os.path.splitext(path_to_shifts)[0])
    pdb_file = os.path.basename(os.path.splitext(path_to_pdb)[0])

    output_dir = os.path.join(os.getcwd(), pdb_file+'_'+shift_file)

    if os.path.isdir(output_dir):
        shutil.rmtree(output_dir)
        check_quiet_print(args.quiet,"overwriting output directory: "+output_dir)
    os.makedirs(output_dir)  

    check_quiet_print(args.quiet,"calculating RCI")
    chain_output_rci = rci_nef.calc_RCI(path_to_shifts,output_dir+'/other_output/',panav=args.reref,quiet=args.quiet)

    check_quiet_print(args.quiet,'extracting models from '+path_to_pdb)
    pdbs,args.oligomer  = rigidipy.extract_pdb(path_to_pdb,output_dir,freeligands=args.lig,nonstandardres=args.nonstd,oligomers=args.olig,quiet=args.quiet)
    if len(pdbs) > 0:
        check_quiet_print(args.quiet,' -> extracted '+str(len(pdbs))+' model(s)')
    else:
        check_quiet_print(args.quiet,' -> CRITICAL ERROR failed to extract any models, exiting')
        sys.exit(1)

    check_quiet_print(args.quiet,'computing the flexibility of each model')
    chain_output_rigidity = {}
    for pdb in sorted(pdbs,key=natsort):
        check_quiet_print(args.quiet," -> "+os.path.basename(os.path.splitext(pdb)[0]),end='')
        if rigidipy.calc_rigidity(pdb,quiet=args.quiet):
            check_quiet_print(args.quiet,' DONE')
            if args.oligomer:
                rigidipy.rigid_decomp(pdb,output_dir+'/other_output/extracted_pdbs/combined/',output_dir+'/other_output/FIRST/')
                try:
                    os.remove(output_dir+'/other_output/extracted_pdbs/combined/decomp_list')  # important to delete decomp_list
                except OSError:
                    pass
            else:
                chain_output_rigidity = rigidipy.rigid_decomp(pdb,output_dir+'/other_output/extracted_pdbs/',output_dir+'/other_output/FIRST/',chain_output=chain_output_rigidity)
                try:
                    os.remove(output_dir+'/other_output/extracted_pdbs/decomp_list')  # important to delete decomp_list
                except OSError:
                    pass
        else:
            check_quiet_print(args.quiet,' FAILED')

    if len(glob.glob(output_dir+'/other_output/FIRST/*.decomp')) == 0: # check FIRST worked
        check_quiet_print(args.quiet,' -> CRITICAL ERROR FIRST failed to run for any models, exiting')
        sys.exit(1)

    if args.oligomer:
        for pdb in glob.glob(output_dir+"/other_output/FIRST/*.decomp"):
            chain_output_rigidity = rigidipy.split_decomp(pdb,pdb_file,output_dir+"/other_output/FIRST/",chain_output_rigidity)
            os.remove(pdb)

        try:
            os.remove("resi_ref.tmp")
        except:
            pass

    decomps = glob.glob(output_dir+'/other_output/FIRST/*.decomp')

    check_quiet_print(args.quiet,'computing ANSURR scores')

    for chain in chain_output_rigidity:
        for rigidity in sorted(chain_output_rigidity[chain],key=natsort):
            if chain in chain_output_rci:
                for rci in sorted(chain_output_rci[chain],key=natsort):
                    compare.compare_rci_rigidity(rigidity,rci,output_dir+'/ANSURR_output/',quiet=args.quiet)
            else:
                for rci_chain in sorted(chain_output_rci,key=natsort):
                    for rci in chain_output_rci[rci_chain]:
                        compare.compare_rci_rigidity(rigidity,rci,output_dir+'/ANSURR_output/',quiet=args.quiet)  


    check_quiet_print(args.quiet,"summary of ANSURR scores")
    for line in open(output_dir+'/ANSURR_output/scores.out','r'):
        check_quiet_print(args.quiet," -> "+line.strip())

    sys.exit(0)

      
if __name__ == "__main__":
    main()




#!/usr/bin/env python

import subprocess
import sys
import os
import json
import shutil
import glob

from importlib import resources
from sys import platform
from collections import Counter

from ansurr.functions import check_quiet_print

def calc_rigidity(pdb,quiet=False):

    if platform == "linux" or platform == "linux2":
        with resources.path("ansurr.bin", "calc_rigidity_gcc4.8.4.bin") as f:  # use this for the package
        #with resources.path("bin", "calc_rigidity_gcc4.8.4.bin") as f: 
            data_file_path = f
    elif platform == "darwin":
        with resources.path("ansurr.bin", "calc_rigidity_OSX.bin") as f:  # use this for the package
        #with resources.path("bin", "calc_rigidity_OSX.bin") as f:
            data_file_path = f
    elif platform == "win32":
        check_quiet_print(quiet,'ANSURR will not yet run on Windows')
        sys.exit(0)

    lib_location = str(os.path.dirname(data_file_path))+'/../'
    
    s = subprocess.run([data_file_path, pdb, lib_location],stdout=subprocess.PIPE, stderr=open(os.devnull, 'w'))
   
    if "Calculating rigidity" in str(s.stdout):
        return True
    else:
        return False

def make_monomers(pdbid,chains,model,resi_ref,output_dir):
    pdb = output_dir+'combined/'+pdbid+chains+'_'+model+'.pdb'
    prev_chain = 'XXX'
    for line in open(pdb,'r'):
        if 'ATOM' in line[:4] or 'HETATM' in line[:6]:
            chain = line[21]
            resi = int(line[22:26])
            orig_resi = resi - resi_ref[model][chain]['new_first'] + resi_ref[model][chain]['orig_first']
            if prev_chain == 'XXX':
                prev_chain = chain
                out = open(output_dir+pdbid+chain+'_'+model+'.pdb','w')
            elif chain != prev_chain:
                out.close()
                out = open(output_dir+pdbid+chain+'_'+model+'.pdb','w')
            out.write(line[0:22]+''.join([' ']*(4-len(str(orig_resi))))+str(orig_resi)+line[26:])
            prev_chain = chain
    out.close()

def parse_pdb_lines(pdb_lines,model,models,freeligands, nonstandardres, oligomers, conects, nonstandard, quiet=False):
    standard_res = ['ALA','CYS','ASP','GLU','PHE','GLY','HIS','ILE','LYS','LEU','MET','ASN','PRO','GLN','ARG','SER','THR','VAL','TRP','TYR']
    selected_pdb_lines = {}

    for chain in pdb_lines:
        std_res = 0
        for line in pdb_lines[chain]:
            resn = line[17:20]
            if resn in standard_res and line[:4] == 'ATOM': # helps diagnose non-standard residues from free ligands, non-standard residues are HETATMS that come with standard residues too
                std_res = 1
                break
        if std_res == 1:
            for line in pdb_lines[chain]:
                resn = line[17:20]
                if resn in standard_res:
                    if chain not in selected_pdb_lines:
                        selected_pdb_lines[chain] = [line]
                    else:
                        selected_pdb_lines[chain].append(line)
                elif nonstandardres:
                    if chain not in selected_pdb_lines:
                        selected_pdb_lines[chain] = ['ATOM  '+line[6:]]
                    else:
                        selected_pdb_lines[chain].append('ATOM  '+line[6:])
                    if resn not in nonstandard:
                        check_quiet_print(quiet," -> non-standard residue "+resn.replace(' ','')+" will be included in rigidity calculations")
                        nonstandard.append(resn)
                else:
                    if resn not in nonstandard:
                        check_quiet_print(quiet," -> found a non-standard residue ("+resn.replace(' ','')+") which are ignored by default. To include non-standard residues, re-run ANSURR with the -n flag")
                        nonstandard.append(resn)
        else:
            for line in pdb_lines[chain]:
                resn = line[17:20]
                if freeligands:
                    if chain not in selected_pdb_lines:
                        selected_pdb_lines[chain] = [line]
                    else:
                        selected_pdb_lines[chain].append(line)
                    if resn not in nonstandard:
                        check_quiet_print(quiet," -> free ligand "+resn.replace(' ','')+" will be included in rigidity calculations")
                        nonstandard.append(resn)
                else:
                    if resn not in nonstandard:
                        check_quiet_print(quiet," -> found a free ligand ("+resn.replace(' ','')+") which are ignored by default. To include free ligands, re-run ANSURR with the -l flag")
                        nonstandard.append(resn)
    for chain in selected_pdb_lines:
        if len(selected_pdb_lines[chain]) > 0:
            if model not in models:  
                models[model] = {chain:selected_pdb_lines[chain]}
            elif chain not in models[model]:
                models[model][chain] = selected_pdb_lines[chain]
            else:
                models[model][chain].extend(selected_pdb_lines[chain])
    return models

    
def extract_pdb(pdb_file, output_dir, freeligands=False, nonstandardres=False, oligomers=False, conects=False,quiet=False):

    pdb_in = open(pdb_file,'r')
    pdb = os.path.basename(os.path.splitext(pdb_file)[0])

    model = '1' # assume first model is model 1
    models = {}

    pdb_lines = {}
    chain = ''
    conect = []
    nonstandard = []

    pdbs_out = []

    conformer_chosen = ''
    conformer_rejects = []

    chain_output = {}

    for line in pdb_in:
        if line[:6] == 'EXPDTA':
            if 'NMR' not in line and 'SOLID' not in line: # some older solution NMR structures just referred to as NMR
                check_quiet_print(quiet,' -> CRITICAL ERROR this version of ANSURR is only for validating structures solved using SOLUTION NMR, exiting')
                quit()
        elif line[:5] == 'MODEL':
            model = line.split()[1]
        elif line[:4] == 'ATOM' or line[:6] == 'HETATM':
            chain = '' if line[21] == ' ' else line[21]
            conformer = '' if line[16] == ' ' else line[16]

            if conformer != '' and conformer_chosen == '':
                conformer_chosen = conformer

            if conformer == '' or conformer == conformer_chosen:
                if chain not in pdb_lines:
                    pdb_lines[chain] = [line]
                else:
                    pdb_lines[chain].append(line)
            else:
                conformer_rejects.append(conformer)

        elif 'TER' in line[:3] or 'END' in line[:3]: # parse after TER to help diagnose free ligands from non-sandard residues
            if len(pdb_lines) > 0:   
                models = parse_pdb_lines(pdb_lines,model,models,freeligands, nonstandardres, oligomers, conects,nonstandard,quiet=quiet)
                pdb_lines = {}
        elif 'CONECT'in line[:6] and conects == True:
            conect.append(line)

    if len(pdb_lines) > 0:   # catch any extra stuff in case structure doesn't end with TER or END, e.g. for CNS output
        models = parse_pdb_lines(pdb_lines,model,models,freeligands, nonstandardres, oligomers, conects,nonstandard,quiet=quiet)
        pdb_lines = {}

    for model in models: # this orders by resi number as sometimes pdbs are not ordered correctly (e.g. pdb 2lrl)
        for chain in models[model]:
            pdb_lines = models[model][chain]
            resi_pdb_lines = {}

            for line in pdb_lines:
                resi = int(line[22:26])
                if resi not in resi_pdb_lines:
                    resi_pdb_lines[resi] = [line]
                else:
                    resi_pdb_lines[resi].append(line) 

            pdb_lines = [resi_pdb_lines[i] for i in sorted(resi_pdb_lines)]
            models[model][chain] =  [item for sublist in pdb_lines for item in sublist]


    chains_done = []
    resi_ref = {}
    old_new_atom_num = {} # 
    for model in models:
        chains = ''
        if oligomers:
            if len(models[model]) > 1:
                if not os.path.exists(output_dir+'/other_output/extracted_pdbs/combined'):
                    os.makedirs(output_dir+'/other_output/extracted_pdbs/combined')
                for chain in models[model]:
                    chains += chain
                out = open(output_dir+'/other_output/extracted_pdbs/combined/'+pdb+chains+'_'+model+'.pdb','w')
                pdbs_out.append(output_dir+'/other_output/extracted_pdbs/combined/'+pdb+chains+'_'+model+'.pdb')
                count = 1
                num = 0
                resi_ref[model] = {}
                for chain in models[model]:
                    prev_resi = -99999
                    resi_ref[model][chain] = {'orig_first':int(models[model][chain][0][22:26]),'orig_last':int(models[model][chain][-1][22:26]),'new_first':'','new_last':''}
                    for l in models[model][chain]:
                        old_new_atom_num[l[6:11].strip()] = str(count)
                        resi = int(l[22:26])
                        if prev_resi != -99999:
                            num += resi - prev_resi
                        else:
                            num += 1
                        prev_resi = resi
                        if resi_ref[model][chain]['new_first'] == '':
                            resi_ref[model][chain]['new_first'] = num
                        out.write(l[0:6]+format(str(count)," >5s")+l[11:22]+''.join([' ']*(4-len(str(num))))+str(num)+l[26:].replace('\n','')+'\n') #this makes sure there is a single new line char at end (needed for FIRST to run correctly)
                        count +=1 
                    resi_ref[model][chain]['new_last'] = num
                out.write('END\n')
                for c in conect:
                    corrected_c = ''
                    for i in c.split()[1:]:
                        if i in old_new_atom_num:
                            corrected_c += ' '+old_new_atom_num[i]
                    if len(corrected_c.split()) > 1:
                        out.write('CONECT '+corrected_c+'\n')
                out.close()
                make_monomers(pdb,chains,model,resi_ref,output_dir+'/other_output/extracted_pdbs/')
                if chains not in chains_done:
                    check_quiet_print(quiet," -> chains "+chains+" combined into a single structure to calculate flexibility ")   
                    chains_done.append(chains)
            else:
                oligomers = False
                check_quiet_print(quiet,' -> found only a single chain ('+str([i for i in models[model]][0])+'), no need to combine')
        if oligomers == False:
            for chain in models[model]:
                chains += chain
                if not os.path.exists(output_dir+'/other_output/extracted_pdbs/'):
                    os.makedirs(output_dir+'/other_output/extracted_pdbs/')
                out = open(output_dir+'/other_output/extracted_pdbs/'+pdb+chain+'_'+model+'.pdb','w')
                pdbs_out.append(output_dir+'/other_output/extracted_pdbs/'+pdb+chain+'_'+model+'.pdb')
                count = 1
                for l in models[model][chain]:
                    old_new_atom_num[l[6:11].strip()] = str(count)
                    out.write(l[0:6]+format(str(count)," >5s")+l[11:])
                    count +=1 
                out.write('END\n')
                for c in conect:
                    corrected_c = ''
                    for i in c.split()[1:]:
                        if i in old_new_atom_num:
                            corrected_c += ' '+old_new_atom_num[i]
                    if len(corrected_c.split()) > 1:
                        out.write('CONECT '+corrected_c+'\n')
                out.close()
            if len(chains) > 1 and chains not in chains_done:
                check_quiet_print(quiet," -> found "+str(len(chains))+" chains, which are extracted seperately by default. To combine chains when calculating flexibility, re-run ANSURR with the -o flag")
                chains_done.append(chains)

    if conformer_chosen != '':
        err_string = " -> found alternative locations, using: "+conformer_chosen+", ignoring: "
        for c in sorted(list(set(conformer_rejects))):
            err_string+=c+', '
        check_quiet_print(quiet,err_string[:-2])

    if len(conect) > 0:
        check_quiet_print(quiet," -> read in "+str(len(conect))+" CONECT records")
    if oligomers:
        json.dump(resi_ref, open("resi_ref.tmp",'w'))

    return pdbs_out, oligomers


def rigid_decomp(path_to_pdb,path_to_decomp,output_dir,chain_output=''):

    pdb = open(path_to_pdb,'r')
    decomp = open(path_to_decomp+'decomp_list','r')

    class res(object):
        _registry = []
        def __init__(self,i,name,atom):
            self._registry.append(self)
            self.i = i
            self.name = name
            self.CA = atom
            self.clusters = []
            self.energy = 0

    # get atom numbers for each CA atom from pdb, also get chain for chain_output_rigidity
    for line in pdb:
        if line[:4] == 'ATOM' or line[:6] == 'HETATM':
            atom_number = int(line[6:11])
            atom_name = line[12:16]
            resn = line[17:20].replace(' ','')
            resi = int(line[22:26])
            chain = line[21]
            if atom_name == ' CA ': # this is occasionally a problem for ligands which have a CA atom, however, it's difficult to spot a ligand CA atom from a non-std residue CA atom, and doesn't effect scoring etc. Leave for now
                r = res(resi,resn,atom_number)

    # import decomp_list         
    data = []
    all_data = []
    energy = []
    rigid_clusters = []
    for line in decomp:
        if 'HEADER' not in line[:6]:
            if 'END' in line[:3]:
                rc = []
                data = data[:atom_number] # remove overflow
                all_data.append(data)
                cluster_count = Counter(data)
                for c in cluster_count:
                    if int(cluster_count[c]) >= 15:
                        rc.append(c)
                rigid_clusters.append(rc)
                data = []
            elif 'A:' in line[:2]:
                energy.append(float(line[10:21]))
            elif 'A:' not in line[:2] and 'B:' not in line[:2]:
                line = line.replace('\n','').split(':')
                for l in line:
                    data.append(int(l))  

    # append rigid clusters to residues
    for d in all_data:
         for r in res._registry:
            r.clusters.append(d[r.CA-1])

    # find rigid clusters
    for x in range(len(all_data)):
        for r in res._registry:
            if r.clusters[x] in rigid_clusters[x]:
                r.energy = energy[x]

    #write output
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)
    out = open(output_dir+os.path.basename(os.path.splitext(path_to_pdb)[0])+'.decomp','w')

    if chain_output != '':
        if chain not in chain_output:
            chain_output[chain] = [output_dir+os.path.basename(os.path.splitext(path_to_pdb)[0])+'.decomp']
        else:
            chain_output[chain].append(output_dir+os.path.basename(os.path.splitext(path_to_pdb)[0])+'.decomp')

    for r in res._registry:
        out.write(str(r.i) + ' ' + r.name +' ' + str(r.energy) + '\n')
    out.close()

    if chain_output != '':
        return chain_output


def split_decomp(decomp,pdb_file,output_dir,chain_output):

    resi_ref = json.load(open("resi_ref.tmp"))

    decomp_num = sys.argv[1].split('/')[-1]
    num = decomp.split('.decomp')[0].split('_')[-1]

    chains = ''
    for chain in resi_ref[num]:
        chains += chain

    for chain in resi_ref[num]:
        out = open(output_dir+pdb_file+chain+'_'+num+'.decomp','w')
        orig_first = resi_ref[num][chain]['orig_first']
        orig_last = resi_ref[num][chain]['orig_last']
        new_first = resi_ref[num][chain]['new_first']
        new_last = resi_ref[num][chain]['new_last']
        for line in open(decomp,'r'):
            resi= int(line.split()[0]) 
            if resi >= new_first and resi <= new_last:
                out.write(str(resi - new_first + orig_first)+ ' '+line.split()[1]+' '+line.split()[2]+'\n')
            if resi == new_last:
                out.close()
                break

        if chain not in chain_output:
            chain_output[chain] = [output_dir+pdb_file+chain+'_'+num+'.decomp']
        else:
            chain_output[chain].append(output_dir+pdb_file+chain+'_'+num+'.decomp')

    return chain_output
    





#!/usr/bin/env python

import pynmrstar
import numpy as np
import sys
import os
import subprocess
import glob

from ansurr import rci_nef_dicts
from ansurr.functions import check_quiet_print

from importlib import resources
from shutil import which
from shutil import move


class cs_set(object):
    _registry = []
    def __init__(self,i):
        self._registry.append(self)
        self.i = i
        self.chains = []
        
class chain(object):
    _registry = []
    def __init__(self,i):
        self._registry.append(self)
        self.i = i
        self.residues = []

class res(object):
    _registry = []
    def __init__(self,i,name):
        self._registry.append(self)
        self.i = i
        self.name = name
        self.shifts = [] 
        self.reref_shifts = [] 
        self.secondary_shifts = []
        self.secondary_shifts_smoothed = []
        self.neighbours = ['',''] 
        self.assumed_RC = 0
        self.shift_types = ''
        self.RCI = 0
        self.RCI_smoothed = 0
        
class atom(object):
    _registry = []
    def __init__(self,atom_type,shift):
        self.atom_type = atom_type
        self.shift = shift

def extract_shifts_PANAV(file_in):
    for line in file_in:
        line = line.split('\t')
        resi = line[0].replace(' ','')[1:]
        try:
            resn = rci_nef_dicts.amino_acids[line[0].replace(' ','')[0]]
            resi = int(resi)
            r = res(resi,resn)
            if line[1] != '      ' or line[2] != '      ' or line[3] != '      ' or line[4] != '      ' or line[5] != '      ' or line[6] != '      ': 
                append_reref_shift(r,'C',line[1])
                append_reref_shift(r,'CA',line[2])
                append_reref_shift(r,'CB',line[3])
                append_reref_shift(r,'N',line[4])
                append_reref_shift(r,'H',line[5])
                append_reref_shift(r,'HA',line[6])
        except:
            pass
          
def append_shift(res, atom_type, shift):
    try:
        shift = float(shift)
        res.shifts.append(atom(atom_type,shift))
    except:
        pass

def append_reref_shift(res, atom_type, shift):
    try:
        shift = float(shift)
        res.reref_shifts.append(atom(atom_type,shift))
    except:
        pass

def is_tool(name):

    return which(name) is not None

def calc_RCI(path_to_nef_file,output_dir='',panav=False,quiet=False):

    entry = pynmrstar.Entry.from_file(path_to_nef_file)  # read NEF format
    
    shift_format = ''

    cs_result_sets=[]
    for chemical_shift_loop in entry.get_loops_by_category("_nef_chemical_shift"): # get sets of chemical shift files
        cs_result_sets.append(chemical_shift_loop.get_tag(['chain_code', 'sequence_code', 'residue_name', 'atom_name', 'value']))
        shift_format = 'nef'

    if len(cs_result_sets) == 0:
        try:
            cs_result_sets=[]
            for chemical_shift_loop in entry.get_loops_by_category("_Atom_chem_shift"):
                cs_result_sets.append(chemical_shift_loop.get_tag(['Entity_ID','seq_ID', 'Comp_ID', 'Atom_ID', 'Val','Auth_asym_ID']))
                shift_format = 'nmrstarv3_authchain'
        except:
            try:
                cs_result_sets=[]
                for chemical_shift_loop in entry.get_loops_by_category("_Atom_chem_shift"):
                    cs_result_sets.append(chemical_shift_loop.get_tag(['Entity_ID','seq_ID', 'Comp_ID', 'Atom_ID', 'Val']))
                    shift_format = 'nmrstarv3'
            except:
                pass

    if len(cs_result_sets) == 0:
        print(' -> no chemical shifts found, exiting')
        sys.exit(1)
    else:

        check_quiet_print(quiet," -> shifts are in "+shift_format.split('_')[0]+" format")
        panav_set_chain_offset = {}
        for chemical_shift_set in enumerate(cs_result_sets):                                  # iterate over sets of shifts
            s = cs_set(chemical_shift_set[0]+1)
            for line in chemical_shift_set[1]:

                if shift_format == 'nef' or shift_format == 'nmrstarv3':
                    chain_name = line[0]
                elif shift_format == 'nmrstarv3_authchain':
                    if line[5].isalpha():
                        chain_name = line[5]
                    else:
                        chain_name = line[0]

                res_i = int(''.join(c for c in line[1] if c.isdigit()))                       # ignore non-numeric characters
                res_name = line[2]
                atom_type = line[3]
                shift_value = float(line[4])

                if res_name in rci_nef_dicts.amino_acids_three_letter and atom_type in rci_nef_dicts.backbone_atoms:   # check that residue is a standard amino acid and atom is a backbone atom

                    if chain_name not in [c.i for c in s.chains]:            # make chain object if chain is new
                        c = chain(chain_name)
                        s.chains.append(c)

                    for c in s.chains:                                       # check if chain has residue already and make residue object if not
                        if c.i == chain_name:
                            if res_i not in [r.i for r in c.residues]:                      
                                r = res(res_i,res_name)
                                c.residues.append(r)

                            for r in c.residues:
                                if r.i == res_i and r.name == res_name:
                                    append_shift(r,atom_type,shift_value)
                                    break

            if panav:
                if is_tool('java'):
                    if not os.path.exists(output_dir+'PANAV/'):
                        os.makedirs(output_dir+'PANAV/')
                    for c in s.chains:
                        out = open(output_dir+'PANAV/'+str(os.path.basename(os.path.splitext(path_to_nef_file)[0]))+"_"+c.i+"_"+str(s.i)+'_forpanav.str','w')

                        panav_set_chain_offset[str(os.path.basename(os.path.splitext(path_to_nef_file)[0]))+"_"+c.i+"_"+str(s.i)+'.panav'] = {'set':s.i,'chain':c.i}
                        old_sequence_ID = ''
                        seq_single_letter = ''
                        lines_shifts = []
                        num = 1
                        offset = ''
                        for r in c.residues:   
                            if offset == '':
                                offset = r.i - 1
                                panav_set_chain_offset[str(os.path.basename(os.path.splitext(path_to_nef_file)[0]))+"_"+c.i+"_"+str(s.i)+'.panav']['offset'] = offset # PANAV requires to start at residue 1
                            for a in r.shifts:
                                lines_shifts.append(str(num)+' '+str(r.i-offset)+' '+r.name+' '+a.atom_type+' '+str(a.shift) + ' 0 1')
                                num+=1
                            
                            x=1        
                            if old_sequence_ID == '':
                                old_sequence_ID = int(r.i) - 1
                            while True:
                                if int(r.i) == old_sequence_ID + x:
                                    seq_single_letter += rci_nef_dicts.amino_acids_three_letter[r.name]
                                    break
                                else:
                                    seq_single_letter += 'X'
                                    x+=1
                            old_sequence_ID = old_sequence_ID + x 
                        
                        out.write("data_99999\nsave_PROTEIN\n")
                        out.write("_Entity.Polymer_seq_one_letter_code "+seq_single_letter+'\n')
                        out.write("   stop_\nloop_\n      _Atom_chem_shift.ID\n      _Atom_chem_shift.Seq_ID\n      _Atom_chem_shift.Comp_ID\n      _Atom_chem_shift.Atom_ID\n      _Atom_chem_shift.Val\n      _Atom_chem_shift.Val_err\n      _Atom_chem_shift.Assigned_chem_shift_list_ID\n")             
                        for line in lines_shifts:
                            out.write(line+'\n')
                        
                        out.close()
                else:
                    check_quiet_print(quiet," -> java is not installed, will proceed without re-referencing shifts")
                    panav = False


        if panav:
            check_quiet_print(quiet," -> re-referencing shifts using PANAV")
            with resources.path("ansurr.bin", "PANAV.jar") as f: 
                panav_file_path = f

            run_panav = subprocess.run(["java", "-jar", panav_file_path, output_dir+'PANAV/'],stdout=subprocess.PIPE, stderr=open(os.devnull, 'w'))

            if len(glob.glob(output_dir+'PANAV/*_forpanav.out_calibrated')) > 0:
                for p in glob.glob(output_dir+'PANAV/*_forpanav.out_calibrated'):
                    panav_output = os.path.basename(os.path.splitext(p)[0]).split("_forpanav")[0]+'.panav'
                    move(p,output_dir+'PANAV/'+panav_output)

                    for line in open(output_dir+'PANAV/'+panav_output,'r'):
                        line = line.split('\t')
                        try:
                            resi = int(line[0].replace(' ','')[1:])
                            if line[1] != '      ' or line[2] != '      ' or line[3] != '      ' or line[4] != '      ' or line[5] != '      ' or line[6] != '      ': 
                                for chemical_shift_set in enumerate(cs_result_sets):  
                                    if panav_set_chain_offset[panav_output]["set"] == chemical_shift_set[0]+1:
                                        for c in s.chains:                                               # iterate over different chains
                                            if panav_set_chain_offset[panav_output]["chain"] == c.i:
                                                for r in c.residues:  
                                                    if resi + panav_set_chain_offset[panav_output]["offset"] == r.i:
                                                        append_reref_shift(r,'C',line[1])
                                                        append_reref_shift(r,'CA',line[2])
                                                        append_reref_shift(r,'CB',line[3])
                                                        append_reref_shift(r,'N',line[4])
                                                        append_reref_shift(r,'H',line[5])
                                                        append_reref_shift(r,'HA',line[6])
                        except:
                            pass

            else:
                check_quiet_print(quiet," -> PANAV failed to run, will proceed without re-referencing shifts")
                panav = False

        if panav:
            for chemical_shift_set in enumerate(cs_result_sets):  
                for c in s.chains:                                               
                    for r in c.residues:  
                        r.shifts = r.reref_shifts
             
        for chemical_shift_set in enumerate(cs_result_sets):  
            for c in s.chains:                                               # iterate over different chains
                for r in c.residues:                                                # append neighbours
                    for r2 in c.residues:                        
                        if (r2.i == r.i-1):
                            r.neighbours[0] = r2
                        if (r2.i == r.i+1):
                            r.neighbours[1] = r2

                    if r.name == 'GLY':                                             # average GLY HA shifts
                        HA = []                                                        
                        for s in r.shifts:
                            if s.atom_type in ['QA','HA1','HA2','HA3','HAx','HAy']:
                                HA.append(s.shift)
                        if HA != []:
                            r.shifts = [s for s in r.shifts if s.atom_type not in ['QA','HA1','HA2','HA3','HAx','HAy']]    
                            append_shift(r,'HA',np.mean(HA))

                for r in c.residues:                                                # compute neighbour corrected secondary shifts, apply mininum values
                    for s in r.shifts:
                        correction = 0
                        for n in enumerate(r.neighbours):
                            if n[1] != '':  
                                correction += rci_nef_dicts.RC_correction[s.atom_type][n[0]][n[1].name] 
                        r.secondary_shifts.append(atom(s.atom_type,max(rci_nef_dicts.min_shift_values[s.atom_type],np.abs(s.shift - (rci_nef_dicts.RC_values[s.atom_type][r.name] + correction)))))

                for r in c.residues:                                                # smooth secondary shifts by averaging with neighbouring shifts
                    for s in r.secondary_shifts:
                        shifts_temp = [s.shift]
                        for n in r.neighbours:
                            if n != '':
                                for ns in n.secondary_shifts:
                                    if s.atom_type == ns.atom_type:
                                        shifts_temp.append(ns.shift)
                        r.secondary_shifts_smoothed.append(atom(s.atom_type,max(0.5,rci_nef_dicts.scaling_factor[s.atom_type] * np.mean(shifts_temp))))

                    atom_list = []
                    weight_hash = ''
                    for s in r.secondary_shifts_smoothed:                           # get atom_types for weight selection 
                        atom_list.append(s.atom_type)
                    atom_list = sorted(atom_list)
                    for a in atom_list:                                             # build weight hash
                        weight_hash += a

                    RCI = 0
                    sum_weights = 0.0
                    for s in r.secondary_shifts_smoothed:
                        RCI += (10.0 * (rci_nef_dicts.RCI_weights[weight_hash][s.atom_type]) * s.shift)   # compute RCI
                        sum_weights += rci_nef_dicts.RCI_weights[weight_hash][s.atom_type]
                    r.RCI = min(sum_weights / RCI,0.2)

                    r.num_shifts = len(atom_list)                                   # count number of shifts for output file
                    r.shift_types = weight_hash                                     # shift types for output file


                min_resi = min(r.i for r in c.residues)                             # apply end correction - see Whishart RCI paper
                max_resi = max(r.i for r in c.residues)

                N_term = [r for r in c.residues if r.i in range(min_resi,min_resi+4)]
                C_term = [r for r in c.residues if r.i in range(max_resi-3,max_resi+1)]

                max_N_term_RCI = max([r.RCI for r in N_term])
                max_C_term_RCI = max([r.RCI for r in C_term])

                max_N_term_pos = [r.i for r in N_term if r.RCI == max_N_term_RCI][0]
                max_C_term_pos = [r.i for r in C_term if r.RCI == max_C_term_RCI][-1]

                for r in N_term:
                    if r.i <= max_N_term_pos:
                        r.RCI = (2*np.abs(max_N_term_RCI - r.RCI)) + r.RCI

                for r in C_term:
                    if r.i >= max_C_term_pos:
                        r.RCI = (2*np.abs(max_C_term_RCI - r.RCI)) + r.RCI


                for r in c.residues:                                                # smooth RCI by averaging ith neighbours
                    RCI = [r.RCI]
                    RCI.extend([n.RCI for n in r.neighbours if n != ''])          
                    r.RCI_smoothed = np.mean(RCI)

                    offset = 0.024                                                  # re-scale RCI based on comparisons to computed rigidity from structures
                    scale = 0.2-0.024
                    r.RCI_smoothed = min(max((r.RCI_smoothed-offset)/scale,0.0),1.0)


        if not os.path.exists(output_dir+'RCI'):
            os.makedirs(output_dir+'RCI')
        valid_chains = {}
        chains_output = {}
        for s in cs_set._registry:
            for c in s.chains:
                out = open(output_dir+'RCI/'+os.path.basename(os.path.splitext(path_to_nef_file)[0])+"_"+c.i+"_"+str(s.i)+".rci",'w')   # splittext!
                rci = []
                x = []
                for r in c.residues:
                    x.append(int(r.i))
                    rci.append(r.RCI_smoothed)
                    out.write('{:>5}'.format(str(r.i)) + '{:>4}'.format(r.name) + ' ' + '{:<20.18f}'.format(r.RCI_smoothed) + '{:>2}'.format(r.num_shifts) +' '+ r.shift_types + '\n')
                out.close()

                if s.i not in valid_chains:
                    valid_chains[s.i] = c.i
                else:
                    valid_chains[s.i] += ', '+c.i

                if c.i not in chains_output:
                    chains_output[c.i] = [output_dir+'RCI/'+os.path.basename(os.path.os.path.splitext(path_to_nef_file)[0])+"_"+c.i+"_"+str(s.i)+".rci"]
                else:
                    chains_output[c.i].append(output_dir+'RCI/'+os.path.basename(os.path.os.path.splitext(path_to_nef_file)[0])+"_"+c.i+"_"+str(s.i)+".rci")

        msg=''
        for s in valid_chains:
            msg+= 'set '+str(s)+' [chain(s) '+valid_chains[s]+'], '
        check_quiet_print(quiet,' -> found ' +str(len(valid_chains))+ ' set(s) of shifts - '+msg[:-2])

    return chains_output



def main():

    calc_RCI(sys.argv[1])

    sys.exit(0)

      
if __name__ == "__main__":
    main()


#!/usr/bin/env python

import os
import sys
import numpy as np
import matplotlib
import json

from matplotlib import pyplot as plt
from scipy.stats import spearmanr
from scipy.stats import percentileofscore
from importlib import resources

from ansurr.functions import check_quiet_print

matplotlib.use('Agg')

def compare_seq(seq1,seq2):
    score = 0
    count = 0
    for s1 in seq1:
        if s1 == seq2[count]:
            score += 1
        count += 1
    return score

def fix_missing_res(d,report_break,quiet=False):
    count = d['resi'][0]
    pos = 0
    to_delete = []
    for i in d['resi']:
        if i > count:
            if report_break == 1:
                check_quiet_print(quiet,'*WARNING break in PDB numbering at resi '+str(count-1)+'* ', end='')
                report_break = 2
            for k in d:
                if k == 'resi':
                    d[k].insert(pos,count)
                elif k == 'resn':
                    d[k].insert(pos,'XXX')
                else:
                    d[k].insert(pos,np.nan)
        elif i < count:
            if report_break == 1:
                check_quiet_print(quiet,'*WARNING unexpected PDB numbering at resi '+str(count-1)+', renumbering* ', end='')
                report_break = 2
            d['resi'][pos] = count   
        elif report_break == 2:
            report_break = 1
        count += 1
        pos += 1
    return d

def calc_score(seq1,seq2):
    score = 0
    count = 0
    for s1 in seq1:
        if s1 == seq2[count]:
            score += 1
        count += 1
    return score

def find_breaks(nums):
    ranges = sum((list(t) for t in zip(nums, nums[1:]) if t[0]+1 != t[1]), [])
    iranges = iter(nums[0:1] + ranges + nums[-1:])
    breaks = [[n,int(next(iranges))+1] for n in iranges]
    return breaks

def align(RCI,FIRST,cut_off=0.5,quiet=False):
    quit_lowseqID = 0
    if len(RCI['resn']) <= len(FIRST['resn']): # "a" is always the shorter seq
        a = RCI['resn']
        b = FIRST['resn']
    else:
        a = FIRST['resn']
        b = RCI['resn']

    max_score_N = 0
    for i in range(len(b)): # start from N
        a_test = a[-i-1:len(b)+1]
        a_test.extend([np.nan]*(len(b)-i-1))
        a_test = a_test[::-1]
        a_test.extend([np.nan]*(len(b)-len(a_test)))
        a_test = a_test[::-1]  
        score = calc_score(a_test,b)
        if score > max_score_N:
            max_score_N = score
            chosen_i = i

    max_score_C = max_score_N # then start from C and look for better fit
    for i in range(1,len(a)):
        a_test = a[:-i]
        a_test = a_test[::-1]
        a_test.extend([np.nan]*(len(b)-len(a_test)))
        a_test = a_test[::-1]   
        score = calc_score(a_test,b)
        if score > max_score_C:
            max_score_C = score
            chosen_i = i

    if max_score_N >= max_score_C:
        if len(RCI['resi']) <= len(FIRST['resi']):
            for r in RCI:  
                RCI[r] = RCI[r][-chosen_i-1:len(b)+1] 
                RCI[r].extend([np.nan]*(len(b)-chosen_i-1))
                RCI[r] = RCI[r][::-1]   
                RCI[r].extend([np.nan]*(len(b)-len(RCI[r])))
                RCI[r] = RCI[r][::-1]
        else:
            for f in FIRST:
                FIRST[f] = FIRST[f][-chosen_i-1:len(b)+1] 
                FIRST[f].extend([np.nan]*(len(b)-chosen_i-1)) 
                FIRST[f] = FIRST[f][::-1]   
                FIRST[f].extend([np.nan]*(len(b)-len(FIRST[f])))
                FIRST[f] = FIRST[f][::-1]
        num_resn = len([i for i in FIRST['resn'] if i != 'XXX'])
        if max_score_N < cut_off * num_resn:
            check_quiet_print(quiet,'sequence identity ('+str(round(100*max_score_N/num_resn,1))+'%) is below cut-off ('+str(100*cut_off)+'%), skipping')
            quit_lowseqID = 1
    else:
        if len(RCI['resi']) <= len(FIRST['resi']):
            for r in RCI:
                RCI[r] = RCI[r][:-chosen_i]
                RCI[r] = RCI[r][::-1]
                RCI[r].extend([np.nan]*(len(b)-len(RCI[r])))
                RCI[r] = RCI[r][::-1]
        else:
            for f in FIRST:
                FIRST[f] = FIRST[f][:-chosen_i]
                FIRST[f] = FIRST[f][::-1]
                FIRST[f].extend([np.nan]*(len(b)-len(FIRST[f])))
                FIRST[f] = FIRST[f][::-1]
        num_resn = len([i for i in FIRST['resn'] if i != 'XXX'])
        if max_score_C < cut_off * num_resn:
            check_quiet_print(quiet,'sequence identity ('+str(round(100*max_score_C/num_resn,1))+'%) is below cut-off ('+str(100*cut_off)+'%), skipping')
            quit_lowseqID = 1
    RCI['resi'] = FIRST['resi']
    return RCI, FIRST, quit_lowseqID

def trim(res1,res2): # lazy variable names fix this!
    s = 0
    e = 0
    sc = 0
    ec = 0
    for i in enumerate(res1):
        if str(i[1]) != 'nan': 
            if sc == 3:
                if i[1] != res2[i[0]]:
                    ec += 1
                else:
                    ec = 0
                if ec == 3:
                    e = i[0]
            else:
                if i[1] == res2[i[0]]:
                    sc += 1
                else:
                    sc = 0
                if sc == 3:
                    s = i[0]
    s = s-2
    if ec == 3:
        e = e-2
    else:
        e = len(res1) - ec
    return s,e
   
def movingaverage(interval, window_size):
    window= np.ones(int(window_size))/float(window_size)
    return np.convolve(interval, window, 'same')

def get_segments(resi):
    prev_res = min(resi) if resi else None
    segments = list()
    for number in enumerate(resi):
        if number[1] != prev_res+1:
            segments.append([number[0]])
        elif len(segments[-1]) > 1:
            segments[-1][-1] = number[0]
        else:
            segments[-1].append(number[0])
        prev_res = number[1]
    return segments

def smooth(resi,data):
    data_smoothed = []
    segs = get_segments(resi)
    for s in segs:
        if len(s) > 2:
            data_smoothed_temp = movingaverage(data[s[0]:s[1]+1],3)
            N = (data[s[0]] + data[s[0]+1]) / 2.0
            data_smoothed_temp[0] = N
            C = (data[s[1]] + data[s[1]-1]) / 2.0
            data_smoothed_temp[-1] = C
            data_smoothed.extend(data_smoothed_temp)
        elif len(s) == 2:
            data_smoothed_temp = movingaverage(data[s[0]:s[1]+1],2)
            N = (data[s[0]] + data[s[0]+1]) / 2.0
            data_smoothed_temp[0] = N
            C = (data[s[1]] + data[s[1]-1]) / 2.0
            data_smoothed_temp[-1] = C
            data_smoothed.extend(data_smoothed_temp)
        else:
            data_smoothed.append(data[s[0]])
    return data_smoothed

def calc_RMSD(RCI_score_for_corr,FIRST_score_for_corr):
    diff = []
    for i in enumerate(RCI_score_for_corr):
        r = i[1]
        f = FIRST_score_for_corr[i[0]]
        diff.append(np.square(r-f))
    return np.sqrt(np.mean(diff))

def rescale_FIRST(FIRST):
    K = 315777.09
    T = 298.15
    Hartree = 0.00038
    FIRST['score'] = [np.exp((4.2* x * Hartree)*K/T) if not np.isnan(x) else np.nan for x in FIRST['score']]
    FIRST_nan = [i for i,x in enumerate(FIRST['score']) if np.isnan(x)]
    FIRST['score'] = smooth([FIRST['resi'][i] for i,x in enumerate(FIRST['score']) if not np.isnan(x)],[i for i in FIRST['score'] if not np.isnan(i)])
    for n in FIRST_nan:
        FIRST['score'].insert(n,np.nan)
    return FIRST['score']

def rescale_RCI(RCI_score): 
    offset = 0.024
    scale = 0.2-0.024
    RCI_score = [min(max((i-offset)/scale,0.0),1.0) for i in RCI_score]
    return RCI_score

def plot(RCI, FIRST):
    run_CYRANGE = "0"
    helices_x = [FIRST['resi'][i[0]] for i in enumerate(FIRST['ss']) if i[1] == 'H'] # FIRST resi can be nan!
    helices_y = [1.04]*len(helices_x)
    strands_x = [FIRST['resi'][i[0]] for i in enumerate(FIRST['ss']) if i[1] == 'E']
    strands_y = [1.04]*len(strands_x)

    RC_residues_x = [FIRST['resi'][i[0]] for i in enumerate(FIRST['resi']) if RCI['shift_types'][i[0]] == 'RC']
    if len(RC_residues_x) > 0:
        RC_residues_y = [1.02]*len(RC_residues_x)
        plt.scatter(RC_residues_x,RC_residues_y,color='black',marker="x",s=5)

    if run_CYRANGE == '1':
        welldefined_x = [FIRST['resi'][i[0]] for i in enumerate(FIRST['welldefined']) if i[1] == '1'] # FIRST resi can be nan!
        welldefined_y = [1.06]*len(welldefined_x)
        illdefined_x = [FIRST['resi'][i[0]] for i in enumerate(FIRST['welldefined']) if i[1] == '0'] # FIRST resi can be nan!
        illdefined_y = [1.06]*len(illdefined_x)
        plt.scatter(welldefined_x,welldefined_y,color='limegreen',s=2)
        plt.scatter(illdefined_x,illdefined_y,color='grey',s=2)

    # get range over which to plot
    resi_range = []
    for i in enumerate(FIRST['resi']):
        if str(FIRST['score'][i[0]]) != 'nan' and str(RCI['score'][i[0]]) != 'nan':
            resi_range.append(i[1])

    plt.plot(RCI['resi'],RCI['score'])
    plt.plot(FIRST['resi'],FIRST['score'])
    plt.scatter(helices_x,helices_y,color='red',s=2)
    plt.scatter(strands_x,strands_y,color='blue',s=2)
    plt.xlabel('residue number',size=10)
    plt.ylabel('flexibility',size=10)
    plt.ylim(-0.025,1.075)
    plt.xlim(min(resi_range)-5,max(resi_range)+5)
    if run_CYRANGE == '1':
        plt.title('structure: ' + PDB_ID + ' shifts: ' + SHIFT_ID + ' shift%: '+av_perc_shifts_out+'\n All residues | Corr: '+"{:.3f}".format(spearman_noRC)+' CorrScore: ' + "{:.1f}".format(corr_score)+' RMSD: '+"{:.3f}".format(RMSD_noRC)+' RMSDScore: ' + "{:.1f}".format(RMSD_score)+'\n Well-defined | Corr: '+"{:.3f}".format(spearman_noRC_welldefined)+' CorrScore: ' + "{:.1f}".format(corr_score_welldefined)+' RMSD: '+"{:.3f}".format(RMSD_noRC_welldefined)+' RMSDScore: ' + "{:.1f}".format(RMSD_score_welldefined),fontsize=10)
    else:
        plt.title('structure: ' + PDB_ID + ' shifts: ' + SHIFT_ID + ' shift%: '+av_perc_shifts_out+'\n Corr: '+"{:.3f}".format(spearman_noRC)+' CorrScore: ' + "{:.1f}".format(corr_score)+' RMSD: '+"{:.3f}".format(RMSD_noRC)+' RMSDScore: ' + "{:.1f}".format(RMSD_score),fontsize=10)
    plt.savefig(PDB_ID+'_'+SHIFT_ID+'.png',dpi=300,bbox_inches='tight')

def all_same(items):
    return all(x == items[0] for x in items)


def compare_rci_rigidity(input_first,input_rci,output_dir='',quiet=False):
    
    # import data
    FIRST_in = open(input_first,'r')
    RCI_in = open(input_rci,'r')
    PDB_ID = os.path.basename(os.path.splitext(input_first)[0])
    SHIFT_ID= os.path.basename(os.path.splitext(input_rci)[0])
    #ANSURR_PATH = sys.argv[4]
    #run_CYRANGE = sys.argv[5]
    run_CYRANGE = "0"

    CHAIN_ID='' # clunky but allows for '_' in PDB name, need to check if this is an issue anywhere else
    for i in PDB_ID.split('_')[:-1]:
        CHAIN_ID+=i+'_'
    CHAIN_ID = CHAIN_ID[-2]

    if run_CYRANGE == '1':
        with open(os.getcwd()+'/other_output/CYRANGE/cyrange.json') as json_file:
            cyrange = json.load(json_file)

    with resources.path("ansurr.lib", "benchmarks.json") as f:
        benchmark_file_path = f

    with open(benchmark_file_path) as json_file:
        benchmarks = json.load(json_file)

    check_quiet_print(quiet," -> "+PDB_ID + '|'+ SHIFT_ID+' ',end='')

    # secondary structure
    """ss_dict = {}
                try:
                    secondary_structure_in = open(sys.argv[3],'r')
                    for line in secondary_structure_in:
                        line = line.split()
                        try:
                            ss_dict[int(line[0])] = line[2]
                        except:
                            ss_dict[int(line[0])] = np.nan
                except:
                    pass"""

    # read in data
    RCI = {'resi':[],'resn':[],'score':[],'shifts':[],'shift_types':[]}
    FIRST = {'resi':[],'resn':[],'score':[],'ss':[]}

    for line in RCI_in:
        line = line.split()
        RCI['resi'].append(int(line[0]))
        RCI['resn'].append(line[1])
        RCI['score'].append(float(line[2]))
        if line[1] == 'GLY' or line[1] == 'PRO':
            RCI['shifts'].append(float(line[3])/5.0)
            RCI['shift_types'].append(line[4])
        else:
            RCI['shifts'].append(float(line[3])/6.0)
            RCI['shift_types'].append(line[4])
            
    for line in FIRST_in:
        line = line.split()
        FIRST['resi'].append(int(line[0]))
        FIRST['resn'].append(line[1])
        FIRST['score'].append(float(line[2]))
        try:
            FIRST['ss'].append(ss_dict[int(line[0])])
        except:
            FIRST['ss'].append(np.nan)

    if run_CYRANGE == '1':
        FIRST['welldefined'] = []
        if len(cyrange) == 1:
            chain = 'default chain'
        else:
            chain = CHAIN_ID

        for resi in FIRST['resi']:
            if resi in cyrange[chain]:
                FIRST['welldefined'].append('1')
            else:
                FIRST['welldefined'].append('0')
    else:
        FIRST['welldefined'] = ['0']*len(FIRST['resi'])
        
    # fix missing residues
    FIRST = fix_missing_res(FIRST,1,quiet=quiet)
    RCI = fix_missing_res(RCI,0,quiet=quiet)

    # align
    RCI,FIRST,quit_lowseqID = align(RCI,FIRST,quiet=quiet)

    if quit_lowseqID == 0:

        # trim - look for where first 3 residues and last 3 residues in aligned RCI/FIRST output are and make these the beginning/end
        start,end = trim(RCI['resn'],FIRST['resn'])
        for r in RCI:
            RCI[r] = RCI[r][start:end] 
        for f in FIRST:
            FIRST[f] = FIRST[f][start:end] 

        # rescale
        FIRST['score'] = rescale_FIRST(FIRST)
        #RCI['score'] = rescale_RCI(RCI['score']) # RCI now arrives rescaled

        # various stats
        RCI_nan = [i for i,x in enumerate(RCI['score']) if np.isnan(x)]
        FIRST_nan = [i for i,x in enumerate(FIRST['score']) if np.isnan(x)]
        RCI_noRC = [x for i,x in enumerate(RCI['score']) if i not in FIRST_nan and not np.isnan(x) and RCI['shift_types'][i] != 'RC']
        FIRST_noRC = [x for i,x in enumerate(FIRST['score']) if i not in RCI_nan and not np.isnan(x) and RCI['shift_types'][i] != 'RC']

        RMSD_noRC =  calc_RMSD(RCI_noRC,FIRST_noRC)
        RMSD_score = 100.0 - percentileofscore(benchmarks['rmsd'],RMSD_noRC)

        if all_same(FIRST_noRC) or all_same(RCI_noRC):
            spearman_noRC = np.nan
            corr_score = np.nan
            check_quiet_print(quiet,'*WARNING Spearman correlation coefficient cannot be determined, setting correlation score to NaN * ',end='')
        else:
            spearman_noRC = spearmanr(RCI_noRC,FIRST_noRC)[0]
            corr_score = percentileofscore(benchmarks['corr'],spearman_noRC)

        if run_CYRANGE == '1':
            RCI_noRC_welldefined = [x for i,x in enumerate(RCI['score']) if i not in FIRST_nan and not np.isnan(x) and RCI['shift_types'][i] != 'RC' and FIRST['welldefined'][i]=='1']
            FIRST_noRC_welldefined = [x for i,x in enumerate(FIRST['score']) if i not in RCI_nan and not np.isnan(x) and RCI['shift_types'][i] != 'RC' and FIRST['welldefined'][i]=='1']
            
            RMSD_noRC_welldefined =  calc_RMSD(RCI_noRC_welldefined,FIRST_noRC_welldefined)
            RMSD_score_welldefined = 100.0 - percentileofscore(benchmarks['rmsd_welldefined'],RMSD_noRC_welldefined)
            #RMSD_score_welldefined = 100.0 - percentileofscore(benchmarks['rmsd_welldefined'],RMSD_noRC_welldefined)

            if all_same(FIRST_noRC_welldefined) or all_same(RCI_noRC_welldefined):
                spearman_noRC_welldefined = np.nan
                corr_score_welldefined = np.nan
                check_quiet_print(quiet,'*WARNING Spearman correlation coefficient for well-defined residues cannot be determined, setting correlation score to NaN * ',end='')
            else:
                spearman_noRC_welldefined = spearmanr(RCI_noRC_welldefined,FIRST_noRC_welldefined)[0]
                corr_score_welldefined = percentileofscore(benchmarks['corr_welldefined'],spearman_noRC_welldefined)
                #corr_score_welldefined = percentileofscore(benchmarks['corr_welldefined'],spearman_noRC_welldefined)


        av_perc_shifts = int(round(np.nanmean([RCI['shifts'][i[0]] for i in enumerate(RCI['resi']) if not np.isnan(i[1])])*100.0))

        if av_perc_shifts < 75:
            av_perc_shifts_out = str(av_perc_shifts)+' (RCI less reliable!)'
            check_quiet_print(quiet,'*WARNING chemical shift completeness (' + str(av_perc_shifts) +'%)' +' is below recommended minimum (75%), RCI may be unreliable* DONE')
        else:
            av_perc_shifts_out = str(av_perc_shifts)
            check_quiet_print(quiet,'DONE')
            
        # write output file
        if not os.path.exists(output_dir):
            os.makedirs(output_dir+'figs')
            os.makedirs(output_dir+'out')

        RCI_FIRST_out = open(output_dir+'out/'+PDB_ID+'_'+SHIFT_ID+'.out','w')
        for i in enumerate(FIRST['resi']):

            ss = str(FIRST['ss'][i[0]])
            wd = str(FIRST['welldefined'][i[0]])
            if ss == 'nan':
                ss = '.'
            if wd == 'nan':
                wd = '.'

            if FIRST['resn'][i[0]] != 'XXX':
                RCI_FIRST_out.write('{:>5}'.format(FIRST['resi'][i[0]])+' '+'{:<4}'.format(FIRST['resn'][i[0]])+'{:<23.20f}'.format(float(RCI['score'][i[0]]))+'{:<23.20f}'.format(float(FIRST['score'][i[0]]))+ss+' '+wd+' '+'{:<5.2f}'.format(RCI['shifts'][i[0]])+'{:<10}'.format(RCI['shift_types'][i[0]])+'\n')
        RCI_FIRST_out.close()

        # append to scores.out
        scores = open(output_dir+'scores.out','a+')
        if run_CYRANGE == '1':
            scores.write('PDB: '+ '{:<11}'.format(PDB_ID) + ' SHIFTS: '+ '{:<11}'.format(SHIFT_ID) + ' SHIFT%: '+ '{:3}'.format(av_perc_shifts) + ' Corr: '+ '{: 5.3f}'.format(spearman_noRC) + ' CorrScore: '+ '{:4.1f}'.format(round(corr_score,1)) + ' RMSD: '+ '{:4.3f}'.format(RMSD_noRC) + ' RMSDScore: '+ '{:4.1f}'.format(round(RMSD_score,1))+ ' Corr-WD: '+ '{: 5.3f}'.format(spearman_noRC_welldefined)+' CorrScore-WD: '+ '{:4.1f}'.format(round(corr_score_welldefined,1)) + ' RMSD-WD: '+ '{:4.3f}'.format(RMSD_noRC_welldefined) + ' RMSDScore-WD: '+ '{:4.1f}'.format(round(RMSD_score_welldefined,1))+'\n')
        else:
            scores.write('PDB: '+ '{:<11}'.format(PDB_ID) + ' SHIFTS: '+ '{:<11}'.format(SHIFT_ID) + ' SHIFT%: '+ '{:3}'.format(av_perc_shifts) + ' Corr: '+ '{: 5.3f}'.format(spearman_noRC) + ' CorrScore: '+ '{:4.1f}'.format(round(corr_score,1)) + ' RMSD: '+ '{:4.3f}'.format(RMSD_noRC)+ ' RMSDScore: '+ '{:4.1f}'.format(round(RMSD_score,1))+ '\n')
        scores.close()

        # plot figs
        #plot(RCI,FIRST)

        helices_x = [FIRST['resi'][i[0]] for i in enumerate(FIRST['ss']) if i[1] == 'H'] # FIRST resi can be nan!
        helices_y = [1.04]*len(helices_x)
        strands_x = [FIRST['resi'][i[0]] for i in enumerate(FIRST['ss']) if i[1] == 'E']
        strands_y = [1.04]*len(strands_x)

        RC_residues_x = [FIRST['resi'][i[0]] for i in enumerate(FIRST['resi']) if RCI['shift_types'][i[0]] == 'RC']
        if len(RC_residues_x) > 0:
            RC_residues_y = [1.02]*len(RC_residues_x)
            plt.scatter(RC_residues_x,RC_residues_y,color='black',marker="x",s=5)

        if run_CYRANGE == '1':
            welldefined_x = [FIRST['resi'][i[0]] for i in enumerate(FIRST['welldefined']) if i[1] == '1'] # FIRST resi can be nan!
            welldefined_y = [1.06]*len(welldefined_x)
            illdefined_x = [FIRST['resi'][i[0]] for i in enumerate(FIRST['welldefined']) if i[1] == '0'] # FIRST resi can be nan!
            illdefined_y = [1.06]*len(illdefined_x)
            plt.scatter(welldefined_x,welldefined_y,color='limegreen',s=2)
            plt.scatter(illdefined_x,illdefined_y,color='grey',s=2)

        # get range over which to plot
        resi_range = []
        for i in enumerate(FIRST['resi']):
            if str(FIRST['score'][i[0]]) != 'nan' and str(RCI['score'][i[0]]) != 'nan':
                resi_range.append(i[1])

        plt.figure()

        plt.plot(RCI['resi'],RCI['score'])
        plt.plot(FIRST['resi'],FIRST['score'])
        plt.scatter(helices_x,helices_y,color='red',s=2)
        plt.scatter(strands_x,strands_y,color='blue',s=2)
        plt.xlabel('residue number',size=10)
        plt.ylabel('flexibility',size=10)
        plt.ylim(-0.025,1.075)
        plt.xlim(min(resi_range)-5,max(resi_range)+5)
        if run_CYRANGE == '1':
            plt.title('structure: ' + PDB_ID + ' shifts: ' + SHIFT_ID + ' shift%: '+av_perc_shifts_out+'\n All residues | Corr: '+"{:.3f}".format(spearman_noRC)+' CorrScore: ' + "{:.1f}".format(corr_score)+' RMSD: '+"{:.3f}".format(RMSD_noRC)+' RMSDScore: ' + "{:.1f}".format(RMSD_score)+'\n Well-defined | Corr: '+"{:.3f}".format(spearman_noRC_welldefined)+' CorrScore: ' + "{:.1f}".format(corr_score_welldefined)+' RMSD: '+"{:.3f}".format(RMSD_noRC_welldefined)+' RMSDScore: ' + "{:.1f}".format(RMSD_score_welldefined),fontsize=10)
        else:
            plt.title('structure: ' + PDB_ID + ' shifts: ' + SHIFT_ID + ' shift%: '+av_perc_shifts_out+'\n Corr: '+"{:.3f}".format(spearman_noRC)+' CorrScore: ' + "{:.1f}".format(corr_score)+' RMSD: '+"{:.3f}".format(RMSD_noRC)+' RMSDScore: ' + "{:.1f}".format(RMSD_score),fontsize=10)
        plt.savefig(output_dir+'figs/'+PDB_ID+'_'+SHIFT_ID+'.png',dpi=300,bbox_inches='tight')

        plt.close()

def main():

    compare_rci_rigidity(sys.argv[1],sys.argv[2])


    sys.exit(0)


      
if __name__ == "__main__":
    main()
